@@ -1,9 +1,9 @@
 /*
- * $Id: dialog.c,v 1.237 2014/02/19 19:49:04 tom Exp $
+ * $Id: dialog.c,v 1.246 2016/02/09 00:05:39 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  Copyright 2000-2013,2014	Thomas E. Dickey
+ *  Copyright 2000-2015,2016	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -151,6 +151,7 @@ typedef enum {
     ,o_editbox
     ,o_fselect
     ,o_timebox
+    ,o_week_start
 #endif
 #ifdef HAVE_XDIALOG2
     ,o_buildlist
@@ -164,6 +165,7 @@ typedef enum {
 #ifdef HAVE_DLG_TRACE
     ,o_trace
 #endif
+    ,o_iso_week
 } eOptions;
 
 /*
@@ -189,6 +191,9 @@ typedef struct {
 static bool *dialog_opts;
 static char **dialog_argv;
 
+static char **special_argv = 0;
+static int special_argc = 0;
+
 static bool ignore_unknown = FALSE;
 
 static const char *program = "dialog";
@@ -320,6 +325,8 @@ static const Options options[] = {
     { "editbox",	o_editbox,		2, "<file> <height> <width>" },
     { "fselect",	o_fselect,		2, "<filepath> <height> <width>" },
     { "timebox",	o_timebox,		2, "<text> <height> <width> <hour> <minute> <second>" },
+    { "week-start",	o_week_start,		1, "<str>" },
+    { "iso-week",	o_iso_week,		1, NULL },
 #endif
 #ifdef HAVE_XDIALOG2
     { "buildlist",	o_buildlist,		2, "<text> <height> <width> <tag1> <item1> <status1>..." },
@@ -414,49 +421,58 @@ unescape_argv(int *argcp, char ***argvp)
 		    blob[length] = '\0';
 
 		    list = dlg_string_to_argv(blob);
-		    if ((added = dlg_count_argv(list)) != 0) {
-			if (added > 2) {
-			    size_t need = (size_t) (*argcp + added + 1);
-			    if (doalloc) {
-				*argvp = dlg_realloc(char *, need, *argvp);
-				assert_ptr(*argvp, "unescape_argv");
-			    } else {
-				char **newp = dlg_malloc(char *, need);
-				assert_ptr(newp, "unescape_argv");
-				for (n = 0; n < *argcp; ++n) {
-				    newp[n] = (*argvp)[n];
-				}
-				*argvp = newp;
-				doalloc = TRUE;
+		    added = dlg_count_argv(list);
+		    if (added > 2) {
+			/* *argcp arguments before the expansion of --file
+			   - 2 for the removal of '--file <filepath>'
+			   + added for the arguments contained in <filepath>
+			   + 1 for the terminating NULL pointer */
+			size_t need = (size_t) (*argcp + added - 1);
+			if (doalloc) {
+			    *argvp = dlg_realloc(char *, need, *argvp);
+			    assert_ptr(*argvp, "unescape_argv");
+			} else {
+			    char **newp = dlg_malloc(char *, need);
+			    assert_ptr(newp, "unescape_argv");
+			    for (n = 0; n < *argcp; ++n) {
+				newp[n] = (*argvp)[n];
 			    }
-			    dialog_opts = dlg_realloc(bool, need, dialog_opts);
-			    assert_ptr(dialog_opts, "unescape_argv");
+			    /* The new array is not NULL-terminated yet. */
+			    *argvp = newp;
+			    doalloc = TRUE;
 			}
-			if (added > 2) {
-			    for (n = *argcp; n >= j + 2; --n) {
-				(*argvp)[n + added - 2] = (*argvp)[n];
-				dialog_opts[n + added - 2] = dialog_opts[n];
-			    }
-			} else if (added == 1) {
-			    for (n = j; n < *argcp; ++n) {
-				(*argvp)[n] = (*argvp)[n + 1];
-				dialog_opts[n] = dialog_opts[n + 1];
-			    }
+			dialog_opts = dlg_realloc(bool, need, dialog_opts);
+			assert_ptr(dialog_opts, "unescape_argv");
+
+			/* Shift the arguments after '--file <filepath>'
+			   right by (added - 2) positions */
+			for (n = *argcp - 1; n >= j + 2; --n) {
+			    (*argvp)[n + added - 2] = (*argvp)[n];
+			    dialog_opts[n + added - 2] = dialog_opts[n];
 			}
-			for (n = 0; n < added; ++n) {
-			    (*argvp)[n + j] = list[n];
-			    dialog_opts[n + j] = FALSE;
+		    } else if (added < 2) {
+			/* 0 or 1 argument read from the included file
+			   -> shift the arguments after '--file <filepath>'
+			   left by (2 - added) positions */
+			for (n = j + added; n + 2 - added < *argcp; ++n) {
+			    (*argvp)[n] = (*argvp)[n + 2 - added];
+			    dialog_opts[n] = dialog_opts[n + 2 - added];
 			}
-			*argcp += added - 2;
-			free(list);
-			--j;	/* force rescan */
 		    }
+		    /* Copy the inserted arguments to *argvp */
+		    for (n = 0; n < added; ++n) {
+			(*argvp)[n + j] = list[n];
+			dialog_opts[n + j] = FALSE;
+		    }
+		    *argcp += added - 2;
+		    (*argvp)[*argcp] = 0;	/* Write the NULL terminator */
+		    free(list);	/* No-op if 'list' is NULL */
+		    /* Force rescan starting from the first inserted argument */
+		    --j;
+		    continue;
 		} else {
 		    dlg_exiterr("Cannot open --file %s", filename);
 		}
-		(*argvp)[*argcp] = 0;
-		++j;
-		continue;
 	    } else {
 		dlg_exiterr("No value given for --file");
 	    }
@@ -1108,10 +1124,10 @@ static const Mode modes[] =
     {o_msgbox,          4, 4, call_msgbox},
     {o_infobox,         4, 4, call_infobox},
     {o_textbox,         4, 4, call_textbox},
-    {o_menu,            7, 0, call_menu},
-    {o_inputmenu,       7, 0, call_inputmenu},
-    {o_checklist,       8, 0, call_checklist},
-    {o_radiolist,       8, 0, call_radiolist},
+    {o_menu,            6, 0, call_menu},
+    {o_inputmenu,       6, 0, call_inputmenu},
+    {o_checklist,       7, 0, call_checklist},
+    {o_radiolist,       7, 0, call_radiolist},
     {o_inputbox,        4, 5, call_inputbox},
     {o_passwordbox,     4, 5, call_passwordbox},
 #ifdef HAVE_DLG_GAUGE
@@ -1285,7 +1301,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright 2000-2013,2014 Thomas E. Dickey",
+	"Copyright 2000-2015,2016 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1641,6 +1657,18 @@ process_common_options(int argc, char **argv, int offset, bool output)
 	    dialog_vars.no_tags = TRUE;
 	    break;
 #endif
+#ifdef HAVE_XDIALOG
+	case o_week_start:
+	    dialog_vars.week_start = optionString(argv, &offset);
+	    break;
+#endif
+	case o_iso_week:
+	    dialog_vars.iso_week = TRUE;
+	    if (dialog_vars.week_start == 0) {	/* Monday is implied */
+		static char default_1st[] = "1";
+		dialog_vars.week_start = default_1st;
+	    }
+	    break;
 	}
 	if (!done)
 	    offset++;
@@ -1656,8 +1684,6 @@ static void
 init_result(char *buffer)
 {
     static bool first = TRUE;
-    static char **special_argv = 0;
-    static int special_argc = 0;
 
     dlg_trace_msg("# init_result\n");
 
@@ -1684,19 +1710,8 @@ init_result(char *buffer)
 	first = FALSE;
     }
 
-    /*
-     * If we are not checking memory leaks, just do the parse of the
-     * environment once.
-     */
     if (special_argv != 0) {
 	process_common_options(special_argc, special_argv, 0, FALSE);
-#ifdef NO_LEAKS
-	free(special_argv[0]);
-	free(special_argv);
-	special_argv = 0;
-	special_argc = 0;
-	first = TRUE;
-#endif
     }
 }
 
@@ -1991,5 +2006,13 @@ main(int argc, char *argv[])
 	(void) refresh();
 	end_dialog();
     }
+#ifdef NO_LEAKS
+    if (special_argv != 0) {
+	free(special_argv[0]);
+	free(special_argv);
+	special_argv = 0;
+	special_argc = 0;
+    }
+#endif
     dlg_exit(retval);
 }
